==> sd-shared-web-3.0.5-sources.jar: systems/dennis/shared/controller/forms/ValidateForm.java <==
package systems.dennis.shared.controller.forms;

import lombok.SneakyThrows;
import systems.dennis.shared.annotations.DataRetrieverDescription;
import systems.dennis.shared.annotations.BeforeAfterDateValidator;
import systems.dennis.shared.annotations.Validation;
import systems.dennis.shared.config.WebContext;
import systems.dennis.shared.entity.DefaultForm;
import systems.dennis.shared.entity.KeyValue;
import systems.dennis.shared.exceptions.ValidationFailedException;
import systems.dennis.shared.pojo_form.ValidationContext;
import systems.dennis.shared.pojo_form.ValidationResult;
import systems.dennis.shared.service.AbstractService;
import systems.dennis.shared.utils.bean_copier.BeanCopier;
import systems.dennis.shared.validation.ValueValidator;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public interface ValidateForm<FORM extends DefaultForm> {

    @SneakyThrows
    default void validate(KeyValue keyValue) {
        var formClass = getClassOfForm();
        var field = BeanCopier.findField(keyValue.getKey(), formClass);

        Map<String, List<ValidationResult>> validationResult = new HashMap<>();
        validateField(keyValue, field, true, validationResult, keyValue.getValue(),  ((Serviceable) this).getServiceClass() );

        checkValidation(validationResult);
    }

    @SneakyThrows
    default void validate(FORM form, boolean edit) {
        validate(form, edit, ((Serviceable) this).getServiceClass() );
        validateDateBefore(form);
    }
    @SneakyThrows
    default void validate(FORM form, boolean edit, Class<? extends AbstractService> serviceable) {
        var formClass = serviceable.getAnnotation(DataRetrieverDescription.class).form();
        var validatedFields = BeanCopier.findAnnotatedFields(form.getClass(), Validation.class);

        Map<String, List<ValidationResult>> validationResult = new HashMap<>();
        for (Field field : validatedFields) {
            validateField(form, field, edit, validationResult, BeanCopier.readValue(form, field), serviceable);
        }

        checkValidation(validationResult);
    }

    default void validateField(Object objectOfForm, Field field, boolean edit,
                               Map<String, List<ValidationResult>> validation, Object valueOfField, Class<? extends AbstractService> serviceClass)
            throws NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {


        var fieldValidation = field.getAnnotation(Validation.class);

        if (fieldValidation != null) {
            List<ValidationResult> validationResults = new ArrayList<>();

            for (Class<? extends ValueValidator> cl : fieldValidation.value()) {
                var validationClass = cl.getConstructor().newInstance();
                field.setAccessible(true);

                var validationResult = validationClass
                        .validate(serviceClass, objectOfForm, field.getName(), valueOfField, edit, getContext());

                validationResults.add(validationResult);
                field.setAccessible(false);
            }
            validation.put(field.getName(), validationResults);
        }
    }

    default void checkValidation(Map<String, List<ValidationResult>> validationResult) {
        ValidationContext context = new ValidationContext();
        context.setData(validationResult);

        if (!context.getValidationMessages().isEmpty()) {
            throw new ValidationFailedException(context);
        }
    }

    default Class<? extends DefaultForm> getClassOfForm() {
        return ((Serviceable) this).getForm();
    }

    default void validateDateBefore(FORM form) {
        Map<String, List<ValidationResult>> result = new HashMap<>();

        var validatedFields = BeanCopier.findAnnotatedFields(form.getClass(), BeforeAfterDateValidator.class);

        for (Field field : validatedFields) {
            String startDateField = field.getAnnotation(BeforeAfterDateValidator.class).startDateField();
            String endDateField = field.getAnnotation(BeforeAfterDateValidator.class).endDateField();

            String startValue = BeanCopier.readValue(form, startDateField);
            String endValue = BeanCopier.readValue(form, endDateField);

            if (startValue == null || endValue == null) {
                result.put(field.getName(), Collections.singletonList(ValidationResult.PASSED));
            } else {
                validateDateRange(field.getName(), startValue, endValue, result);
            }
        }
        checkValidation(result);
    }

    private void validateDateRange(String fieldName, String startValue, String endValue, Map<String, List<ValidationResult>> result) {
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy");

            Date startDate = dateFormat.parse(startValue);
            Date endDate = dateFormat.parse(endValue);

            if (startDate.after(endDate)) {
                result.put(fieldName, Collections.singletonList(ValidationResult.fail("date.validation.start_date_after_end_date")));
            } else {
                result.put(fieldName, Collections.singletonList(ValidationResult.PASSED));
            }
        } catch (ParseException e) {
            result.put(fieldName, Collections.singletonList(ValidationResult.fail("date.validation.invalid_date_format")));
        }
    }

    WebContext.LocalWebContext getContext();
}

==> sd-shared-web-3.2-sources.jar: systems/dennis/shared/controller/forms/ValidateForm.java <==
package systems.dennis.shared.controller.forms;

import lombok.SneakyThrows;
import systems.dennis.shared.annotations.BeforeAfterDateValidator;
import systems.dennis.shared.annotations.DataRetrieverDescription;
import systems.dennis.shared.annotations.Validation;
import systems.dennis.shared.config.WebContext;
import systems.dennis.shared.entity.KeyValue;
import systems.dennis.shared.exceptions.ValidationFailedException;
import systems.dennis.shared.form.AbstractForm;
import systems.dennis.shared.pojo_form.ValidationContext;
import systems.dennis.shared.pojo_form.ValidationResult;
import systems.dennis.shared.service.AbstractService;
import systems.dennis.shared.utils.bean_copier.BeanCopier;
import systems.dennis.shared.validation.ValueValidator;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;


public interface ValidateForm<FORM extends AbstractForm> {

    @SneakyThrows
    default void validate(KeyValue keyValue) {
        var formClass = getClassOfForm();
        var field = BeanCopier.findField(keyValue.getKey(), formClass);

        Map<String, List<ValidationResult>> validationResult = new HashMap<>();
        validateField(keyValue, field, true, validationResult, keyValue.getValue(),  ((Serviceable) this).getServiceClass() );

        checkValidation(validationResult);
    }

    @SneakyThrows
    default void validate(FORM form, boolean edit) {
        validate(form, edit, ((Serviceable) this).getServiceClass() );
        validateDateBefore(form);
    }
    @SneakyThrows
    default void validate(FORM form, boolean edit, Class<? extends AbstractService> serviceable) {
        var formClass = serviceable.getAnnotation(DataRetrieverDescription.class).form();
        var validatedFields = BeanCopier.findAnnotatedFields(form.getClass(), Validation.class);

        Map<String, List<ValidationResult>> validationResult = new HashMap<>();
        for (Field field : validatedFields) {
            validateField(form, field, edit, validationResult, BeanCopier.readValue(form, field), serviceable);
        }

        checkValidation(validationResult);
    }

    default void validateField(Object objectOfForm, Field field, boolean edit,
                               Map<String, List<ValidationResult>> validation, Object valueOfField, Class<? extends AbstractService> serviceClass)
            throws NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {


        var fieldValidation = field.getAnnotation(Validation.class);

        if (fieldValidation != null) {
            List<ValidationResult> validationResults = new ArrayList<>();

            for (Class<? extends ValueValidator> cl : fieldValidation.value()) {
                var validationClass = cl.getConstructor().newInstance();
                field.setAccessible(true);

                var validationResult = validationClass
                        .validate(serviceClass, objectOfForm, field.getName(), valueOfField, edit, getContext());

                validationResults.add(validationResult);
                field.setAccessible(false);
            }
            validation.put(field.getName(), validationResults);
        }
    }

    default void checkValidation(Map<String, List<ValidationResult>> validationResult) {
        ValidationContext context = new ValidationContext();
        context.setData(validationResult);

        if (!context.getValidationMessages().isEmpty()) {
            throw new ValidationFailedException(context);
        }
    }

    default Class<? extends AbstractForm> getClassOfForm() {
        return ((Serviceable) this).getForm();
    }

    default void validateDateBefore(FORM form) {
        Map<String, List<ValidationResult>> result = new HashMap<>();

        var validatedFields = BeanCopier.findAnnotatedFields(form.getClass(), BeforeAfterDateValidator.class);

        for (Field field : validatedFields) {
            String startDateField = field.getAnnotation(BeforeAfterDateValidator.class).startDateField();
            String endDateField = field.getAnnotation(BeforeAfterDateValidator.class).endDateField();

            String startValue = BeanCopier.readValue(form, startDateField);
            String endValue = BeanCopier.readValue(form, endDateField);

            if (startValue == null || endValue == null) {
                result.put(field.getName(), Collections.singletonList(ValidationResult.PASSED));
            } else {
                validateDateRange(field.getName(), startValue, endValue, result);
            }
        }
        checkValidation(result);
    }

    private void validateDateRange(String fieldName, String startValue, String endValue, Map<String, List<ValidationResult>> result) {
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy");

            Date startDate = dateFormat.parse(startValue);
            Date endDate = dateFormat.parse(endValue);

            if (startDate.after(endDate)) {
                result.put(fieldName, Collections.singletonList(ValidationResult.fail("date.validation.start_date_after_end_date")));
            } else {
                result.put(fieldName, Collections.singletonList(ValidationResult.PASSED));
            }
        } catch (ParseException e) {
            result.put(fieldName, Collections.singletonList(ValidationResult.fail("date.validation.invalid_date_format")));
        }
    }

    WebContext.LocalWebContext getContext();
}
